<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://tonicpow.com
 * @since      1.0.0
 *
 * @package    Tonicpow
 * @subpackage Tonicpow/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Tonicpow
 * @subpackage Tonicpow/includes
 * @author     TonicPow <support@tonicpow.com>
 */
class Tonicpow
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Tonicpow_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	private static $instance;

	/**
	 * @return mixed
	 */
	public static function getInstance()
	{
		if ( ! self::$instance ) {
			self::$instance = new Tonicpow();
		}

		return self::$instance;
	}

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if ( defined( 'TONICPOW_VERSION' ) ) {
			$this->version = TONICPOW_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'tonicpow';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		if (!defined('TONICPOW_DEBUG')) {
            if (get_option('tncpw_debug')) {
                define('TONICPOW_DEBUG', true);
            } else {
                define('TONICPOW_DEBUG', false);
            }
        }
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Tonicpow_Loader. Orchestrates the hooks of the plugin.
	 * - Tonicpow_i18n. Defines internationalization functionality.
	 * - Tonicpow_Admin. Defines all hooks for the admin area.
	 * - Tonicpow_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-tonicpow-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-tonicpow-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-tonicpow-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-tonicpow-public.php';

		$this->loader = new Tonicpow_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Tonicpow_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$plugin_i18n = new Tonicpow_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{

		$plugin_admin = new Tonicpow_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_menu' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'init' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'setup_sections' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'setup_fields' );
		$this->loader->add_action( 'wp_ajax_tonicpow_test_goal', $plugin_admin, 'test_goal_trigger' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Tonicpow_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_public, 'init' );

		if (class_exists( 'WooCommerce' )) {
		    $this->loader->add_action( 'woocommerce_checkout_create_order', $plugin_public, 'woocommerce_checkout_create_order' );
		}

		// Register dynamic hook
		$goals = get_option('tonicpow_goals') ?: [];
		foreach($goals as $goal) {
		    $hook_name = $goal['hook_name'];
            if ($hook_name) {
                switch ( $hook_name ) {
                    case "media_upload_video":
                    case "media_upload_file":
                    case "media_upload_image":
                        $this->loader->add_action( $hook_name, $plugin_public, 'dynamicHook', 10, 1 );
                        break;
                    case "comment_post":
                        $this->loader->add_action( $hook_name, $plugin_public, 'dynamicHook', 10, 2 );
                        break;
                    case "wp_login":
                    case 'woocommerce_payment_complete':
                        $this->loader->add_action( $hook_name, $plugin_public, 'dynamicHook', 10, 3 );
                        break;
                    case 'woocommerce_add_to_cart':
                        $this->loader->add_action( $hook_name, $plugin_public, 'dynamicHook', 10, 6 );
                        break;
                    default:
                        $this->loader->add_action( $hook_name, $plugin_public, 'dynamicHook' );
                }
            }
		}
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 * @since     1.0.0
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return    Tonicpow_Loader    Orchestrates the hooks of the plugin.
	 * @since     1.0.0
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 * @since     1.0.0
	 */
	public function get_version()
	{
		return $this->version;
	}

	public function session( $key = false, $value = false )
	{
		global $wp_session;

		if ( $key !== false ) {
			$wp_session[ $key ] = $value;
		}

		return $wp_session;
	}
}
